#include <signal.h>
#include <stdio.h>
#include <errno.h>

/* This program shows how signals can be blocked and unblocked.

   Try "kill -USR1 <pid>" on the console, where <pid> is the process id.

   Certain system calls (getchar, read, wait, ...) may be
   interrrupted by signals before completing their normal task.
   For example, in this program, getchar() will be interrupted
   by a SIGUSR1 when the latter is unblocked.

   Whether or not a signal interrupts these system calls is configurable
   in the handler; uncomment the line containing SA_RESTART to prevent
   SIGUSR1 from interrupting getchar().

   See also signal (7), section "Interruption of system calls".
*/

void handler ()
{
	printf("coucou\n");
}

int main ()
{
	sigset_t newmask;
	struct sigaction action;

	// uncomment the following line to prevent getchar() from being
	// interrupted by USR1 signals.
	// action.sa_flags = SA_RESTART;

	action.sa_handler = handler;
	sigaction(SIGUSR1,&action,NULL);

	printf("my pid is %d\n",getpid());

	sigemptyset(&newmask);
        sigaddset(&newmask,SIGUSR1);

	while (1)
	{
		char c;
		sigprocmask(SIG_BLOCK, &newmask, NULL);
		printf("SIGUSR1 now BLOCKED, press enter to unblock\n");
		c = getchar();
		printf("c = %d, errno = %d\n",c,errno);

		sigprocmask(SIG_UNBLOCK, &newmask, NULL);
		printf("SIGUSR1 now UNBLOCKED, press enter to block\n");

		// getchar() will be interrrupted if a signal intervenes,
		// and returns -1 in this case.
		c = getchar();
		printf("c = %d, errno = %d\n",c,errno);
	}
}
